/****************************************\
 * $YH-2018-12-16 15:59:36
 * $DATA-HANDLE 
 * 描述：数据处理库
 * 用途：数据的判断于验证
\****************************************/

define(function() {

	// 正则表达式
	var STRNULL_REGEXP = /\s+/g; // 空格验证
	// 数据类型常量
	var objType = {
		OBJECT: "object",
		BOOLEAN: "boolean",
		NUMBER: "number",
		STRING: "string",
		LIST: "list",
		NULL: "null",
		DOM: "dom"
	};

	/**
	 * 数据空验证
	 * @param {Object} obj 验证对象
	 * @return {Boolean} true[为空],false[不为空]
	 */
	function isNull(obj) {
		if(isNullType(obj)) {
			return true;
		} else if(isStringType(obj)) {
			return Object.is("", obj.replace(STRNULL_REGEXP, ""));
		}
		return false;
	}
	/**
	 * 数据不为空验证
	 * @param {Object} obj
	 * @return {Boolean} true[不为空],false[为空]
	 */
	function isNotNull(obj) {
		return !isNull(obj);
	}
	/**
	 * 数据类型判断
	 * @param {Object} obj 数据
	 * @return {String} 数据类型
	 */
	function isType(obj) {
		var type = typeof obj; // 获取数据类型
		if(Object.is(type, "object")) { // 区分'object'类型的具体类型
			if(null == obj) {
				return objType.NULL;
			} else if(obj.nodeType === 1) {
				return objType.DOM;
			} else if(obj instanceof Array) {
				return objType.LIST;
			} else {
				return type;
			}
		} else if(Object.is(type, "undefined")) {
			return objType.NULL;
		} else {
			return type;
		}
	}

	/**
	 * 判断是否为对象
	 * @param {Object} obj
	 * @return {Boolean} true[是],false[不是]
	 */
	function isObjType(obj) {
		return Object.is(isType(obj), objType.OBJECT);
	};
	/**
	 * 判断是否为布尔值
	 * @param {Object} obj
	 * @return {Boolean} true[是],false[不是]
	 */
	function isBooleanType(obj) {
		return Object.is(isType(obj), objType.BOOLEAN);
	};
	/**
	 * 判断是否为数字
	 * @param {Object} obj
	 * @return {Boolean} true[是],false[不是]
	 */
	function isNumberType(obj) {
		return Object.is(isType(obj), objType.NUMBER);
	};
	/**
	 * 判断是否为字符串
	 * @param {Object} obj
	 * @return {Boolean} true[是],false[不是]
	 */
	function isStringType(obj) {
		return Object.is(isType(obj), objType.STRING);
	};
	/**
	 * 判断是否为数组
	 * @param {Object} obj
	 * @return {Boolean} true[是],false[不是]
	 */
	function isListType(obj) {
		return Object.is(isType(obj), objType.LIST);
	};
	/**
	 * 判断是否为空对象类型
	 * @param {Object} obj
	 * @return {Boolean} true[是],false[不是]
	 */
	function isNullType(obj) {
		return Object.is(isType(obj), objType.NULL);
	};
	/**
	 * 判断是否为节点对象
	 * @param {Object} obj
	 * @return {Boolean} true[是],false[不是]
	 */
	function isDomType(obj) {
		return Object.is(isType(obj), objType.DOM);
	};
	return {
		isNull: isNull,
		isNotNull: isNotNull,
		isObjType: isObjType,
		isListType: isListType,
		isNumberType: isNumberType,
		isStringType: isStringType,
		isDomType: isDomType
	}
})